@@ -4,6 +4,7 @@ from pathlib import Path
 from pprint import pformat
 from random import randint
 from typing import Any
+from typing import Callable
 from typing import Dict
 from typing import List
 from typing import Union
@@ -17,7 +18,8 @@ class DB:
         # An in memory copy of the db
         self._db: Dict[str, Dict[str, Any]] = {}
         self._verify = verify_data
-        self._keys = keys
+        self._keys = sorted(keys)
+        self._id_generator = self._generate_id
 
     @typing.no_type_check
     def __repr__(self) -> str:
@@ -39,11 +41,17 @@ class DB:
         """Store the current instance of the DB in a file"""
         self._dump_db_to_json(filename)
 
+    def set_id_generator(self, func: Callable[[], str]) -> None:
+        self._id_generator = func
+
+    def id_exists(self, _id: str) -> bool:
+        return _id in self._db
+
     def add(self, data: Dict[str, Any]) -> str:
         """Add a value to the DB"""
 
         if self._verify_data(data):
-            _id = self._generate_id()
+            _id = str(self._id_generator())
             self._db[_id] = data
             return _id
         return "0"
@@ -51,11 +59,12 @@ class DB:
     def add_many(self, data: List[Dict[str, Any]]) -> None:
         """Add more than one value to the DB at a time"""
 
-        for d in data:
-            self._verify_data(d)
+        if self._verify:
+            for d in data:
+                self._verify_data(d)
 
         for d in data:
-            self._db[self._generate_id()] = d
+            self._db[str(self._id_generator())] = d
 
     def get_by_id(self, _id: str) -> Union[None, Dict[str, Any]]:
         """Get the value from the DB based on the _id"""
@@ -84,6 +93,14 @@ class DB:
         self.delete_by_id(_id)
         return data
 
+    def values(self, count: int = 5, last: bool = False) -> Dict[str, Dict[str, Any]]:
+        if not last:
+            keys = list(self._db)[:count]
+        else:
+            keys = list(self._db)[-count:]
+
+        return {i: self._db[i] for i in keys}
+
     def update_by_id(self, _id: str, data: Dict[str, Any]) -> None:
         """Update a value by it id"""
         if self._db:
@@ -130,10 +147,10 @@ class DB:
     ###############################################################################################
 
     def _generate_id(self) -> str:
-        _id = randint(int("1" "" * 19), int("9" * 20))
+        _id = randint(int("1" + "0" * 19), int("9" * 20))
 
-        while _id in self._db:
-            _id = randint(int("1" "" * 19), int("9" * 20))
+        while str(_id) in self._db:
+            _id = randint(int("1" + "0" * 19), int("9" * 20))
 
         return str(_id)
 
@@ -161,7 +178,7 @@ class DB:
          as provided in the keys list"""
 
         if self._verify:
-            if sorted(self._keys) == sorted(list(data)):
+            if self._keys == sorted(data.keys()):
                 return True
 
             else:
